import type { Pager } from '#/global'
import { apiGetMenuListTree } from '@/api/sys/menu'
import type { RoleDto, RoleInfo } from '@/api/sys/model/roleModel'
import {
  apiDeleteRole,
  apiGetRole,
  apiGetRoleList,
  apiGetRolePer,
  apiInsertRole,
  apiRolePerSave,
  apiUpdateRole
} from '@/api/sys/role'
import { message } from 'ant-design-vue'
import type { Rule } from 'ant-design-vue/es/form'
import { computed, onMounted, reactive, ref, unref } from 'vue'
import type { VxeTableInstance } from 'vxe-table'

const rules: Record<string, Rule[]> = {
  roleName: [{ required: true, message: '请输入角色名称', trigger: 'blur' }],
  roleCode: [{ required: true, message: '请输入角色编码', trigger: 'blur' }]
}
const initPagination: Pager = {
  currentPage: 1,
  pageSize: 10,
  total: 0
}
const initFormState = (): RoleDto & { roleId: string } => {
  return {
    roleId: '',
    orgCode: '',
    roleCode: '',
    roleDes: '',
    roleName: ''
  }
}

export function useRolePage() {
  const tableRef = ref<VxeTableInstance<RoleInfo>>()
  const tableData = ref<RoleInfo[]>([])
  const pagination = reactive<Pager>(initPagination)
  const loading = ref(false)

  /**
   * TODO:查询未添加查询条件
   */
  const queryData = async (page?: number) => {
    loading.value = true
    try {
      page && (pagination.currentPage = page)
      const { result } = await apiGetRoleList()
      tableData.value = result.records
      pagination.total = result.total
    } catch (e) {}
    loading.value = false
  }

  return {
    tableRef,
    tableData,
    pagination,
    loading,
    queryData
  }
}

export function useRoleForm() {
  const roleFormRef = ref()
  const roleFormRules = reactive<Record<string, Rule[]>>(rules)
  const roleFormState = reactive<RoleDto & { roleId: string }>(initFormState())
  const addLoading = ref(false)

  const validate = async () => await roleFormRef.value.validate()
  const resetFrom = () => Object.assign(unref(roleFormState), initFormState())
  const isModify = computed(() => !!roleFormState.roleId)

  /**
   * 根据id查询角色信息
   * @param id 角色id
   */
  const getRoleById = async (id: string) => {
    const { result } = await apiGetRole({ id })
    Object.assign(unref(roleFormState), result)
  }

  /**
   * 删除角色
   * @param id 删除角色的id
   */
  const deleteRole = async (id: string) => {
    try {
      await apiDeleteRole({ ids: id })
      message.success('删除成功!')
    } catch (e) {}
  }

  /**
   * 添加角色
   */
  const addRole = async () => {
    try {
      addLoading.value = true
      await validate()
      isModify.value ? await apiUpdateRole(roleFormState) : await apiInsertRole(roleFormState)
      message.success('添加成功!')
    } catch (e) {}
    addLoading.value = false
  }


  return {
    roleFormRef,
    roleFormRules,
    roleFormState,
    addLoading,
    addRole,
    deleteRole,
    getRoleById,
    resetFrom
  }
}

export function useAuth() {
  const expandedKeys = ref<string[]>([])
  const selectedKeys = ref<string[]>([])
  const checkedKeys = ref<string[]>([])
  const isModal = ref(false)
  const treeLoading = ref(false)
  const treeData = ref<any[]>([])
  const fieldNames = reactive({
    children: 'childNode',
    title: 'permissionName',
    key: 'permissionId'
  })
  onMounted(async () => {
    const { result } = await apiGetMenuListTree()
    treeData.value = result
  })

  const saveRolePer = async (data) => {
    await apiRolePerSave(data)
    isModal.value = false
    message.success('保存成功！')
  }

  const getCurrentRolePer = async (roleId) => {
    treeLoading.value = true
    try {
      const { result } = await apiGetRolePer({roleId})
    checkedKeys.value = result
    } catch {}
    treeLoading.value = false
  }

  return {
    expandedKeys,
    selectedKeys,
    checkedKeys,
    treeData,
    isModal,
    getCurrentRolePer,
    saveRolePer,
    treeLoading,
    fieldNames
  }
}
